'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = testStore;

var _redux = require('redux');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _once = require('lodash/once');

var _once2 = _interopRequireDefault(_once);

var _every = require('lodash/every');

var _every2 = _interopRequireDefault(_every);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function testStore(store) {
  var callback = arguments.length <= 1 || arguments[1] === undefined ? function () {} : arguments[1];

  var actions = [];
  var finalCallback = (0, _once2.default)(callback);

  function testMiddleware(_ref) {
    var dispatch = _ref.dispatch;
    var getState = _ref.getState;

    return function (next) {
      return function (action) {
        if (typeof action !== 'function') {
          actions.push(action);
        }

        next(action);
      };
    };
  }

  var newStore = (0, _redux.applyMiddleware)(_reduxThunk2.default, testMiddleware)(function () {
    return store;
  })();

  newStore.queuedActions = [];

  newStore.when = function (actionType) {
    var assertion = arguments.length <= 1 || arguments[1] === undefined ? function () {} : arguments[1];

    newStore.queuedActions.push({
      type: actionType,
      tested: false,
      assertion: assertion
    });

    return newStore;
  };

  newStore.subscribe(function () {
    var state = newStore.getState();
    var lastAction = actions[actions.length - 1];

    newStore.queuedActions.forEach(function (action, index) {
      if (action.type !== lastAction.type || action.tested) return action;

      var assertion = action.assertion;

      var result = assertion(state, lastAction, actions);

      newStore.queuedActions[index].tested = true;
    });

    if ((0, _every2.default)(newStore.queuedActions, { tested: true })) {
      return finalCallback();
    }
  });

  return newStore;
}